package com.qmth.themis.business.templete.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.ExamStudentDtoImport;
import com.qmth.themis.business.entity.TBExamInvigilateUser;
import com.qmth.themis.business.entity.TBUser;
import com.qmth.themis.business.entity.TBUserRole;
import com.qmth.themis.business.service.TBExamInvigilateUserService;
import com.qmth.themis.business.service.TBUserRoleService;
import com.qmth.themis.business.service.TBUserService;
import com.qmth.themis.business.templete.TaskImportCommon;
import com.qmth.themis.business.templete.TaskImportTemplete;
import com.qmth.themis.business.util.ExcelCallback;
import com.qmth.themis.business.util.ExcelError;
import com.qmth.themis.business.util.ExcelUtil;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 考场导入任务
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/15
 */
public class TaskRoomCodeImportTemplete implements TaskImportTemplete {
    private final static Logger log = LoggerFactory.getLogger(TaskRoomCodeImportTemplete.class);

    /**
     * 考场导入模版
     *
     * @param map
     * @return
     * @throws IOException
     * @throws InterruptedException
     */
    @Override
    @Transactional
    public Result importTask(Map<String, Object> map) throws IOException {
        TaskImportCommon taskImportCommon = new TaskImportCommon(map);
        taskImportCommon.init();
        File file = taskImportCommon.getUploadFile();
        List<LinkedMultiValueMap<Integer, Object>> finalList = ExcelUtil.excelReader(new FileInputStream(file), Lists.newArrayList(ExamStudentDtoImport.class), new ExcelCallback() {
            @Override
            public List<LinkedMultiValueMap<Integer, Object>> callback(List<LinkedMultiValueMap<Integer, Object>> finalList, List<LinkedMultiValueMap<Integer, String>> finalColumnNameList) throws IllegalAccessException {
                List<ExcelError> excelErrorList = new ArrayList<>();
                for (int i = 0; i < finalList.size(); i++) {
                    LinkedMultiValueMap<Integer, Object> map = finalList.get(i);
                    List<Object> examStudentDtoImportList = map.get(i);
                    for (int y = 0; y < examStudentDtoImportList.size(); y++) {
                        ExamStudentDtoImport examStudentDtoImport = (ExamStudentDtoImport) examStudentDtoImportList.get(y);
                        List<ExcelError> excelErrorTemp = ExcelUtil.checkExcelField(examStudentDtoImport, y, i);
                        if (excelErrorTemp.size() > 0) {
                            excelErrorList.addAll(excelErrorTemp);
                        }
                    }
                }
                if (excelErrorList.size() > 0) {
                    throw new BusinessException(JSONObject.toJSONString(excelErrorList));
                }
                return finalList;
            }
        });
        //保存数据到数据库
        if (Objects.nonNull(finalList) && finalList.size() > 0) {
            log.info("开始导入考场数据");
            long start = System.currentTimeMillis();

            TBUserService tbUserService = SpringContextHolder.getBean(TBUserService.class);
            TBUserRoleService tbUserRoleService = SpringContextHolder.getBean(TBUserRoleService.class);
            TBExamInvigilateUserService tbExamInvigilateUserService = SpringContextHolder.getBean(TBExamInvigilateUserService.class);
            String timeFormat = taskImportCommon.getTimeFormat();

            List<TBUser> tbUserList = new ArrayList<>();
            List<TBUserRole> tbUserRoleList = new ArrayList<>();
            List<TBExamInvigilateUser> tbExamInvigilateUserList = new ArrayList<>();
            List<String> txtList = new ArrayList();
            txtList.add(DateUtil.format(new Date(), timeFormat) + "->开始准备处理导入的考场数据");
            int y = 0, min = 0;
            try {
//                Gson gson = new Gson();
                //保存用户
                for (int i = 0; i < finalList.size(); i++) {
                    LinkedMultiValueMap<Integer, Object> finalMap = finalList.get(i);
                    List<Object> examStudentDtoImportList = finalMap.get(i);
                    min = 0;
                    int max = SystemConstant.MAX_IMPORT_SIZE, size = examStudentDtoImportList.size();
                    if (max >= size) {
                        max = size;
                    }
                    while (max <= size) {
                        List subList = examStudentDtoImportList.subList(min, max);
                        for (y = 0; y < subList.size(); y++) {

                        }
                        txtList = taskImportCommon.progress(max, min, size, txtList);
                        if (max == size) {
                            break;
                        }
                        min = max;
                        max += SystemConstant.MAX_IMPORT_SIZE;
                        if (max >= size) {
                            max = size;
                        }
                    }
                }
                long end = System.currentTimeMillis();
                log.info("导入考场数据结束,============耗时============:{}秒", (end - start) / 1000);
            } catch (Exception e) {
                e.printStackTrace();
                txtList = taskImportCommon.exception(min, y, e, txtList);
                if (Objects.nonNull(tbUserList) && tbUserList.size() > 0) {
                    tbUserService.removeByIds(tbUserList.stream().map(s -> s.getId()).collect(Collectors.toList()));
                }
                if (Objects.nonNull(tbUserRoleList) && tbUserRoleList.size() > 0) {
                    tbUserRoleService.removeByIds(tbUserRoleList.stream().map(s -> s.getId()).collect(Collectors.toList()));
                }
                if (Objects.nonNull(tbExamInvigilateUserList) && tbExamInvigilateUserList.size() > 0) {
                    tbExamInvigilateUserService.removeByIds(tbExamInvigilateUserList.stream().map(s -> s.getId()).collect(Collectors.toList()));
                }
                if (e instanceof BusinessException) {
                    throw new BusinessException(e.getMessage());
                } else {
                    throw new RuntimeException(e);
                }
            } finally {
                //这里写入txt文件
                taskImportCommon.writeImportResultTxt(txtList.toString());
            }
        }
        return ResultUtil.ok(JacksonUtil.parseJson(SystemConstant.SUCCESS));
    }
}
